#include <stdio.h>
#include <Xm/List.h>

Widget  appshell;
Widget  ListWidgetID;

Arg args[20];

XmString FirstItem, MiddleItem, LastItem;

Dimension Spacing = 0;
/****************
 *
 * These are the strings that are displayed by the list. They are
 * converted into compound strings at run time by the main program.
 *
 ****************/
static char  *CharListItems[24] = {
 "New Item List",
 "New Selected Items",
 "Maximum Visible",
 "Five Visible",
 "New Selection Policy",
 "New Spacing",
 "New Font",
 "Add Item at Top",
 "Add Item in Middle",
 "Add Item at End",
 "Delete 'Middle Item'",
 "Delete Item at End",
 "Select Top Item",
 "Select Bottom Item",
 "Deselect Top Item",
 "Deselect Bottom Item",
 "Deselect All",
 "Make 5th Item Top",
 "Make 1st Item Top",
 "Make 5th Item Bottom",
 "Make Last Item Bottom",
 "AutoSelect ON",
 "AutoSelect OFF",
 "Quit"
 };

#define NUM_LIST_ITEMS 24

XmString ListItems[NUM_LIST_ITEMS];

/****************
 *
 * This is the second set of items that is displayed by the list.  This
 * set is displayed when "New Selected Items" is activated.
 *
 ****************/
static char *CharNewListItems[7] = {
 "Old Item List",
 "New List item 2",
 "New List item 3",
 "New List item 4",
 "New List item 5",
 "New List item 6",
 "New List item 7"};
 
#define NUM_NEW_LIST_ITEMS 7
XmString NewListItems[NUM_NEW_LIST_ITEMS];

/****************
 *
 * These are the items that are initially selected (highlighted) by the list.
 *
 ****************/
static char *CharSelectedItems[2] = {
 "New Item List",
 "New Selection Policy"};

#define NUM_SELECTED_ITEMS 2
XmString SelectedItems[NUM_SELECTED_ITEMS];

XmStringCharSet cs = "ISOLatin1";

XFontStruct *fnt;
XmFontList font1, font2, font3, curfont;

/************************************************************************
 *          *
 * DoubleClickProc is the XmDEFAULT_ACTION callback.  It functions as a  *
 * big case statement, comparing the item that was double-clicked to  *
 * the items in the list.  When it finds a match, it takes the  *
 * appropriate action.        *
 *          *
 ************************************************************************/
void  DoubleClickProc(w,closure,call_data)
  Widget w;
  XtPointer  closure, call_data;
{
  int j;
  XmListCallbackStruct *cb =  (XmListCallbackStruct *)call_data;
  int  i = 0;
  unsigned char k;
  char *new_selection_policy;

  
  if (XmStringCompare(cb->item,ListItems[0]))  /* Set a new item list */
  {
     XtSetArg(args[i], XmNitems, NewListItems); i++;
     XtSetArg(args[i], XmNitemCount, NUM_NEW_LIST_ITEMS); i++;  
  }
  
  if (XmStringCompare(cb->item,NewListItems[0]))  /* Set the original Item List */
  {
     XtSetArg(args[i], XmNitems, ListItems); i++;
     XtSetArg(args[i], XmNitemCount, NUM_LIST_ITEMS); i++;  
  }

  if (XmStringCompare(cb->item,ListItems[1]))  /* Set a new Selected Item List */
  {
  XtSetArg(args[i], XmNselectedItems, SelectedItems); i++;
  XtSetArg(args[i], XmNselectedItemCount, NUM_SELECTED_ITEMS); i++;
  }

  if (XmStringCompare(cb->item,ListItems[2]))  /* Make all items visible by */
  {                           /* getting the current item  */
  XtSetArg(args[0], XmNitemCount, &j);      /* count and making that the */
  XtGetValues(w, args, 1);            /* visible item count. */
  XtSetArg(args[i], XmNvisibleItemCount, j); i++;
  }

  if (XmStringCompare(cb->item,ListItems[3]))  /* Make five items visible */
  {
  XtSetArg(args[i], XmNvisibleItemCount, 5); i++;
  }

  if (XmStringCompare(cb->item,ListItems[4]))  /* Set a new selection policy */
  {
  XtSetArg(args[0], XmNselectionPolicy, &k);
  XtGetValues(w, args, 1);
  if (k == XmSINGLE_SELECT) {
   k = XmMULTIPLE_SELECT;
   new_selection_policy = "Multiple Selection";
   }
   else
    if (k == XmMULTIPLE_SELECT) {
     k = XmBROWSE_SELECT;
       new_selection_policy = "Browse Selection";
    }
    else
     if (k == XmBROWSE_SELECT) {
      k = XmEXTENDED_SELECT;
    new_selection_policy = "Extended Selection";
      }
     else
    if (k == XmEXTENDED_SELECT) {
       k = XmSINGLE_SELECT;
       new_selection_policy = "Single Selection";
       }
  XtSetArg(args[i], XmNselectionPolicy, k); i++;
  printf ("The new selection policy is %s.\n", new_selection_policy);
  }
  
  if (XmStringCompare(cb->item,ListItems[5]))  /* Increase the spacing between */
  {                                            /* items            */
  Spacing += 2;
  XtSetArg(args[i], XmNlistSpacing, Spacing); i++;
  }
  
  if (XmStringCompare(cb->item,ListItems[6]))  /* Change the font the items */
  {                                            /* are displayed in */
  if (curfont == font1) 
      curfont = font2;
  else 
      if (curfont == font2) 
        curfont = font3;
    else 
        curfont = font1;
  XtSetArg(args[i], XmNfontList, curfont); i++;
  }

  if (XmStringCompare(cb->item,ListItems[21]))  /* Set automatic selection ON */
  {
  XtSetArg(args[i], XmNautomaticSelection, TRUE); i++;
  }

  if (XmStringCompare(cb->item,ListItems[22]))  /* Set automatic selection OFF */
  {
  XtSetArg(args[i], XmNautomaticSelection, FALSE); i++;
  }
/****************
 *
 * If we have set any arguments, do the SetValues and return.
 *
 ****************/
  if (i > 0)  
  {                
    XtSetValues(w,args,i);
    return;
  }
  
  if (XmStringCompare(cb->item,ListItems[7]))  /* Add an item at the first */
  {                           /* position in the list   */
  XmListAddItem(w,FirstItem,1);
  }

  if (XmStringCompare(cb->item,ListItems[8]))  /* Add an item at the last  */
  {                           /* position in the list   */
  XmListAddItem(w,MiddleItem,12);
  }

  if (XmStringCompare(cb->item,ListItems[9]))  /* Add an item at the fifth */
  {                           /* position in the list   */
  XmListAddItem(w,LastItem,0);
  }

  if (XmStringCompare(cb->item,ListItems[10]))  /* Delete the 'Middle Item' */
  {                           /* list element       */
  XmListDeleteItem(w, MiddleItem);
  }

  if (XmStringCompare(cb->item,ListItems[11]))  /* Delete the last item   */
  {
  XmListDeletePos(w,0);
  }

  if (XmStringCompare(cb->item,ListItems[12]))  /* Select the first item */
  {
    XmListSelectPos(w,1,TRUE);
  }

  if (XmStringCompare(cb->item,ListItems[13]))  /* Select the last item */
  {
    XmListSelectPos(w,0,TRUE);
  }

  if (XmStringCompare(cb->item,ListItems[14]))  /* Deselect the first item */
  {
    XmListDeselectPos(w,1);
  }

  if (XmStringCompare(cb->item,ListItems[15]))  /* Deselect the last item */
  {
    XmListDeselectPos(w,0);
  }

  if (XmStringCompare(cb->item,ListItems[16]))  /* Deselect all selected items */
  {
    XmListDeselectAllItems(w);
  }
  
  if (XmStringCompare(cb->item,ListItems[17]))  /* Make the fifth item the top */
  {
    XmListSetPos(w,5);
  }

  if (XmStringCompare(cb->item,ListItems[18]))  /* Make the first item the top */
  {
    XmListSetPos(w,1);
  }

  if (XmStringCompare(cb->item,ListItems[19]))  /* Make the fifth item bottom */
  {
    XmListSetBottomPos(w,5);
  }

  if (XmStringCompare(cb->item,ListItems[20]))  /* Make the last item the bottom */
  {
    XmListSetBottomPos(w,0);
  }

  if (XmStringCompare(cb->item,ListItems[23]))  /* End the program. */
  {
    exit(0);
  }

}

/************************************************************************
 *          *
 * These are the selection callbacks for each type of selection.  They  *
 * simply print a message indicating what type of selection occurred.  *
 *          *
 ************************************************************************/
void  SelectProc(w,closure,call_data)
  Widget w;
  XtPointer  closure, call_data;
{

  printf("Single Selection.\n");

}

void  MultiProc(w,closure,call_data)
  Widget w;
  XtPointer  closure, call_data;
{

  printf("Multiple Selection.\n");

}

void  ExtendProc(w,closure,call_data)
  Widget w;
  XtPointer  closure, call_data;
{

  printf("Extended Selection.\n");

}

void  BrowseProc(w,closure,call_data)
  Widget w;
  XtPointer  closure, call_data;
{

  printf("Browse Selection.\n");

}

void main(argc, argv)
  unsigned int argc;
  char **argv;
{
  int i;
  XtAppContext app_context;
       
/****************
 *
 * Initialize the Xt Intrinsics and create the appshell shell
 *
 ****************/
  appshell = XtAppInitialize(&app_context, "Listdemo", NULL, 0, &argc, argv, NULL, args, 0);

  i = 0;
  
/****************
 *
 * Create the arrays of compound strings from the character arrays.
 *
 ****************/
  for(i = 0; i < NUM_LIST_ITEMS; i++)
  ListItems[i] = (XmString )XmStringCreateLtoR(CharListItems[i],cs);
  for(i = 0; i < NUM_SELECTED_ITEMS; i++)
  SelectedItems[i] = (XmString )XmStringCreateLtoR(CharSelectedItems[i],cs);
  for(i = 0; i < NUM_NEW_LIST_ITEMS; i++)
  NewListItems[i] = (XmString )XmStringCreateLtoR(CharNewListItems[i],cs);

/****************
 *
 * Load the X font structures, and create the necessary fontlists.
 *
 ****************/
  fnt = XLoadQueryFont(XtDisplay(appshell), "vr-20");
  font1 = XmFontListCreate(fnt, cs);
  fnt = XLoadQueryFont(XtDisplay(appshell), "vri-25");
  font2 = XmFontListCreate(fnt, XmFONTLIST_DEFAULT_TAG);
  fnt = XLoadQueryFont(XtDisplay(appshell), "vbee-36");
  font3 = XmFontListCreate(fnt, "FooSet");
  curfont = font1;

/****************
 *
 * Create the individual items to add to the list.  These items are added
 * when the appropriate "Add Item .." is activated.
 *
 ****************/
  FirstItem = XmStringCreateLtoR("++Added Top Item++",cs);
  MiddleItem = XmStringCreateLtoR("++Added Middle Item++", cs);
  LastItem = XmStringCreateLtoR("++Added Bottom Item++",cs);

/****************
 *
 * Create the scrolled list. We will give it the selected items now, and
 * add in the item list later.
 *
 ****************/
  i = 0;
  XtSetArg(args[i], XmNfontList, curfont); i++;
  XtSetArg(args[i], XmNselectedItems, SelectedItems); i++;
  XtSetArg(args[i], XmNselectedItemCount, NUM_SELECTED_ITEMS); i++;
  XtSetArg(args[i], XmNvisibleItemCount, NUM_LIST_ITEMS); i++;  
  XtSetArg(args[i], XmNitems, ListItems); i++;  
  XtSetArg(args[i], XmNitemCount, NUM_LIST_ITEMS); i++;

  ListWidgetID = XmCreateScrolledList(appshell, "ListWidget", args, i);

  XtManageChild(ListWidgetID);

/****************
 *
 * Add the callbacks.
 *
 ****************/
  XtAddCallback(ListWidgetID, XmNsingleSelectionCallback, SelectProc);
  XtAddCallback(ListWidgetID, XmNdefaultActionCallback, DoubleClickProc);
  XtAddCallback(ListWidgetID, XmNmultipleSelectionCallback, MultiProc);
  XtAddCallback(ListWidgetID, XmNextendedSelectionCallback, ExtendProc);
  XtAddCallback(ListWidgetID, XmNbrowseSelectionCallback, BrowseProc);

/****************
 *
 *  realize the hierarchy and call the event processing loop.
 *
 ****************/
  XtRealizeWidget(appshell);
  XtAppMainLoop(app_context);
}
